from ctypes import *
import comtypes
import sys

# Simple sample to communicate with a Burerkt MFC device by using the 00667888_000000_KONF_SW_MFC_DRIVER_DLL.dll.
# There are two parameters  supported:
# first parameter:          The used COM port. 1 for COM1
# second parameter:         The devices used short address

# Following tools was used for this example
# 1)    Activepython (e.g. 2.7.1.3)
#       http://www.activestate.com/activepython/downloads
# 2)    comtypes 0.6.2.win32.exe
#       http://sourceforge.net/projects/comtypes/files/comtypes/0.6.2/

# Before starting this sample:
# Copy the 00667888_000000_KONF_SW_MFC_DRIVER_DLL.dll from the DLL directory into the working directory and rename it to MfcDriver.dll

class MFC_DRIVER:

    def __init__(self):
        # windll wasn't able to use the original DLL name "00667888_000000_KONF_SW_MFC_DRIVER_DLL"
        # It isn't allowed that the dll name includes numbers or underscores
        # so it was renamed  to "MfcDriver"
        try:
            self.libb = windll.MfcDriver
            self.DllLoaded = True
        except:
            print
            print "Copy the 00667888_000000_KONF_SW_MFC_DRIVER_DLL.dll from the DLL directory into the working directory and rename it to MfcDriver.dll"
            self.DllLoaded = False             
            
    def connect(self,port):
        self.port   =  c_int(port)
        #ret = self.libb.OpenComPort(self.port)  # return value 0: function was successfull
        #if ret!=0:
        #    raise "Unable to open COM port: ",ret             
        #return ret
        return self.libb.OpenComPort(self.port)  # return value 0: function was successfull
        
    def close(self):    
           return self.libb.CloseComPort(self.port)            

    def GetNominalFlow(self, ShortAddress, IndexOfGas): 
        _ShortAddress = c_byte( ShortAddress )  # devices address for communication 0..32
        _IndexOfGas  = c_int(IndexOfGas)        # 1..(2)
        _FloatValue  = c_float(0)
        _Unit   = comtypes.BSTR(u'')  
        _Medium = comtypes.BSTR(u'')
        ret = self.libb.GetNominalFlow( self.port, _ShortAddress ,_IndexOfGas, byref(_FloatValue), byref(_Unit), byref(_Medium))
        if ret!=0:
            raise "Error occured during GetNominalFlow: ",ret             
        return _FloatValue.value,_Unit.value,_Medium.value        

    def ExtSetpoint( self, ShortAddress, ExtSetpointActive, SetPointInPerCent ):
        _ShortAddress = c_byte(ShortAddress)            # devices address for communication 0..32
        _ExtSetpointActive = c_byte(ExtSetpointActive)  # 0 := internal setpoint is in use.
                                                        # 1 := external setpoint is used
        _SetPointInPerCent = c_float(SetPointInPerCent) # 0...100.0 external setpoint in %
        ret = self.libb.ExtSetpoint( self.port, _ShortAddress, _ExtSetpointActive, _SetPointInPerCent )
        if ret!=0:
            raise "Error occured during ExtSetpoint: ",ret             
   
    def GetValue(self, ShortAddress, VarName, ArrayIndex):
        _ShortAddress   = c_byte(ShortAddress)  # devices address for communication 0..32
        _VarName   = comtypes.BSTR(VarName)     # The variables description as string
        _ArrayIndex  = c_int(ArrayIndex)        # array index 0..
        _FloatValue  = c_float(0)               # The variables value [out]
        _Unit   = comtypes.BSTR(u'')            # The variables unit [out]
        ret = self.libb.GetValue(self.port, _ShortAddress, byref(_VarName), _ArrayIndex, byref(_FloatValue), byref(_Unit) )
        if ret!=0:
            raise "Error occured during GetValue: ",ret       
        return _FloatValue.value,_Unit.value

    def ReadPrimaryVariable(self, ShortAddress):        # returns the actual flow in %
        _ShortAddress   = c_byte(ShortAddress)          # devices address for communication 0..32
        _PrimaryVariable  = c_float(0)           # ActualValueInPerCent_x [out]
        ret = self.libb.ReadPrimaryVariable(self.port, _ShortAddress, byref(_PrimaryVariable) )
        if ret!=0:
            raise "Error occured during ReadPrimaryVariable: ",ret       
        return _PrimaryVariable.value

    def ReadCurrentAndFourDynamicVariables(self, ShortAddress):
        _ShortAddress   = c_byte(ShortAddress)          # devices address for communication 0..32
        _ActualValueInPerCent_x             = c_float(0)
        _SetpointInPerCent_w                = c_float(0)
        _ManipulatedSetpointInPerCent_y2    = c_float(0)
        _TimerValueInSecond                 = c_float(0)
        ret = self.libb.ReadCurrentAndFourDynamicVariables( self.port, _ShortAddress, byref(_ActualValueInPerCent_x), byref(_SetpointInPerCent_w), byref(_ManipulatedSetpointInPerCent_y2), byref(_TimerValueInSecond) )
        if ret != 0:
            raise "Error occured during ReadCurrentAndFourDynamicVariables: ", ret
        return _ActualValueInPerCent_x.value, _SetpointInPerCent_w.value, _ManipulatedSetpointInPerCent_y2.value, _TimerValueInSecond.value

if __name__ == "__main__":

    port = 1    # default COM port
    Address = 0 # devices default address
    if len(sys.argv) > 1:        
        # take over the first parameter as COM port
        port = sys.argv[1]  
        if len(sys.argv) > 2:
            # take over the second parameter as address
            Address = int(sys.argv[2])

    mfc = MFC_DRIVER()          # instance to the MFC device class
    if mfc.DllLoaded != True:
        print "Dll couldn't be loaded."
    else:
        if 0 != mfc.connect(int(port)):      # connect the COM port with the mfc device and open it
            print
            print "Unable to open COM port: COM", port
        else:
            print 
            # read the nominal flow rate of the specified device and gas
            ActualFlow, Unit, Medium = mfc.GetNominalFlow( Address, 1 )    # short address, index of gas
            print "Nominal Flow                    : ",ActualFlow , Unit , Medium

            # activate the external setpoint
            ExtSetpointActive = 1   # 0 := internal setpoint is in use.
                                    # 1 := external setpoint is used
            ExtSetpoint = 15        # 0...100.0 external setpoint in %
            mfc.ExtSetpoint( Address, ExtSetpointActive, ExtSetpoint )    

            # read the actual flow in %
            ActualValueInPerCent_x = mfc.ReadPrimaryVariable( Address )
            print "ActualValueInPerCent_x          : ", ActualValueInPerCent_x, " %"

            # read process variables setpoint(w), ctual value(x) and manipulated variable(y2)
            ActualValueInPerCent_x, SetpointInPerCent_w, ManipulatedSetpointInPerCent_y2, TimerValueInSecond = mfc.ReadCurrentAndFourDynamicVariables( Address)
            print
            print "SetpointInPerCent_w             : ",SetpointInPerCent_w , " %"
            print "ActualValueInPerCent_x          : ", ActualValueInPerCent_x, " %"
            print "ManipulatedSetpointInPerCent_y2 : ",ManipulatedSetpointInPerCent_y2 , " %"
            print "TimerValueInSecond              : ",TimerValueInSecond , " s"
            
            # use interal setpoint e.g. standard signal input like 4..20mA
            mfc.ExtSetpoint( Address, 0, 0 )    
            # close the instance and unlock the used COM port
            mfc.close() 
            

